import type { ThemeRegistration } from '@shikijs/types'
import React, { useCallback, useEffect, useMemo, useState } from 'react'
import { ScrollView, type ScrollViewProps, type StyleProp, StyleSheet, Text, type TextStyle, View } from 'react-native'
import { ensureHighlighter, getStyleForToken, type HighlighterToken, isLanguageRegistered } from '../utils'

const styles = StyleSheet.create({
  container: {
    padding: 16,
  },
  line: {
    flexDirection: 'row',
    flexWrap: 'wrap',
  },
})

export interface SyntaxHighlighterProps {
  children: string
  language: string
  theme: ThemeRegistration
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
}

export function SyntaxHighlighter({ children, language, theme, textStyle, scrollViewProps }: SyntaxHighlighterProps) {
  const [isReady, setIsReady] = useState(false)
  const [tokens, setTokens] = useState<HighlighterToken[][]>([])

  const highlightCode = useCallback(async () => {
    if (!isLanguageRegistered(language)) {
      console.warn(`Language "${language}" has not been registered. Please register it using registerLanguage()`)
      return
    }

    try {
      const instance = await ensureHighlighter()
      await instance.loadTheme(theme)

      const newTokens = instance
        .codeToTokensBase(children, {
          theme,
          lang: language,
        })
        .map((line, lineIndex) =>
          line.map((token, tokenIndex) => ({
            ...token,
            id: `${lineIndex}-${tokenIndex}-${token.content}`,
          })),
        )

      setTokens(newTokens)
      setIsReady(true)
    }
    catch (error) {
      console.error('Failed to highlight code:', error)
    }
  }, [children, language, theme])

  useEffect(() => {
    highlightCode()
  }, [highlightCode])

  const contentContainerStyle = useMemo(() => [styles.container, scrollViewProps?.contentContainerStyle], [scrollViewProps?.contentContainerStyle])

  if (!isReady) {
    return null
  }

  return (
    <ScrollView {...scrollViewProps} horizontal contentContainerStyle={contentContainerStyle}>
      <View onStartShouldSetResponder={() => true}>
        {tokens.map(line => (
          <View key={`line-${line.map(token => token.content).join('')}`} style={styles.line}>
            {line.map(token => (
              <Text key={token.id} style={[textStyle, getStyleForToken(token)]}>
                {token.content}
              </Text>
            ))}
          </View>
        ))}
      </View>
    </ScrollView>
  )
}
