import type { LanguageRegistration } from '@shikijs/types'
import type { TextStyle } from 'react-native'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'

// Highlighter singleton management
let highlighter: Awaited<ReturnType<typeof createHighlighterCore>> | null = null
const registeredLanguages = new Map<string, LanguageRegistration>()

// Highlighter initialization
export async function ensureHighlighter() {
  if (!highlighter) {
    highlighter = await createHighlighterCore({
      themes: [],
      langs: [],
      engine: createJavaScriptRegexEngine(),
    })
  }
  return highlighter
}

// Language registration
export function registerLanguage(name: string, grammar: Partial<LanguageRegistration>) {
  if (!registeredLanguages.has(name)) {
    // Ensure all required fields are present
    const languageRegistration: LanguageRegistration = {
      name,
      scopeName: grammar.scopeName || `source.${name}`,
      patterns: grammar.patterns || [],
      repository: grammar.repository || {},
      ...grammar,
    }

    registeredLanguages.set(name, languageRegistration)

    if (highlighter) {
      highlighter.loadLanguageSync(languageRegistration)
    }
  }
}

export function isLanguageRegistered(name: string): boolean {
  return registeredLanguages.has(name)
}

export interface HighlighterToken {
  content: string
  color?: string
  fontStyle?: number
  offset: number
  id?: string
}

// Token styling utility
export function getStyleForToken(token: HighlighterToken): TextStyle {
  const styles: TextStyle = {}

  if (token.color) {
    styles.color = token.color
  }

  if (token.fontStyle) {
    if (token.fontStyle & 1)
      styles.fontWeight = 'bold'
    if (token.fontStyle & 2)
      styles.opacity = 0.5 // dim
    if (token.fontStyle & 4)
      styles.fontStyle = 'italic'
    if (token.fontStyle & 8)
      styles.textDecorationLine = 'underline'
  }

  return styles
}
